//  cg-operators.swift
//
//  Created by Michael Kofler on 10.05.16.
//  Copyright © 2016 Michael Kofler. All rights reserved.
//

// Funktionen und Operatoren, um mit CGFloat, CGPoint 
// und CGRect besser umgehen zu können

import SpriteKit

// Pi
let pi = CGFloat(M_PI)

// Pythagoras: a^2 + b^2 = c^2
// c = pythagoras(a, b)
public func pythagoras(a:CGFloat, _ b:CGFloat) -> CGFloat {
  return sqrt(a*a + b*b)
}

// Random
public func cgrand() -> CGFloat {
  return CGFloat(drand48())
}

// y = minMax(x, lower, upper) stellt sicher, dass lower <= y <= upper gilt
public func minMax<T : Comparable>(value: T, minimum: T, maximum: T) -> T {
  return max(min(value, maximum), minimum)
}


// Operatoren für CGPoint-Daten
// CGPoints addieren/subtrahieren
public func + (left: CGPoint, right: CGPoint) -> CGPoint {
  return CGPoint(x: left.x + right.x, y: left.y + right.y)
}
public func += (inout left: CGPoint, right: CGPoint) {
  left = left + right
}
public func - (left: CGPoint, right: CGPoint) -> CGPoint {
  return CGPoint(x: left.x - right.x, y: left.y - right.y)
}
public func -= (inout left: CGPoint, right: CGPoint) {
  left = left - right
}


// CGPoint skalieren
public func * (point: CGPoint, scalar: CGFloat) -> CGPoint {
  return CGPoint(x: point.x * scalar, y: point.y * scalar)
}
public func *= (inout point: CGPoint, scalar: CGFloat) {
  point = point * scalar
}


// CGPoint aus CGRect
public extension CGRect {
  // Mittelpunkt
  func middlePoint() -> CGPoint {
    return CGPoint(x: CGRectGetMidX(self), y: CGRectGetMidY(self))
  }
  // links unten im iOS-Koordinatensystem
  func minPoint() -> CGPoint {
    return CGPoint(x: CGRectGetMinX(self), y: CGRectGetMinY(self))
  }
  // rechts oben im iOS-Koordinatensystem
  func maxPoint() -> CGPoint {
    return CGPoint(x: CGRectGetMaxX(self), y: CGRectGetMaxY(self))
  }
}

public extension CGPoint {
  // passt die Koordinaten des Punkts so an, dass dieser sich
  // innerhalb des Rechtecks befindet
  func withinRect(rect: CGRect) -> CGPoint {
    return CGPoint(x: min(CGRectGetMaxX(rect), max(CGRectGetMinX(rect), self.x)),
                   y: min(CGRectGetMaxY(rect), max(CGRectGetMinY(rect), self.y)))
  }
  
  // Länge
  func length() -> CGFloat {
    return pythagoras(self.x,  self.y)
  }
  
  // Abstand
  func distanceTo(other: CGPoint) -> CGFloat {
    return pythagoras(self.x-other.x, self.y-other.y)
  }
}
